<?php if ( ! defined('BASEPATH')) exit('No direct script access allowed');

/**
 * Template de controller
 */
class Example extends CI_Controller {

	function __construct()
	{
		parent::__construct();
		log_message('debug', 'Class '.__CLASS__.' initialized.');
		$this->controller = strtolower(__CLASS__);
	}

	/**
	 * Remap
	 *
	 * Recebe todas as chamadas de funções deste controller e procede conforme
	 * o seguinte:
	 * - Se não for passada nenhuma função, chama p_index.
	 * - Se for passada alguma função que existe nesta classe, chama ela e
	 * repassa os parâmetros originais.
	 * - Se for passado qualquer outro valor, chama p_exibir passando
	 * o valor como parâmetro.
	 */
	function _remap($method = FALSE, $params = array())
	{
		if (method_exists($this, 'p_'.$method))
		{
			return call_user_func_array(array($this, 'p_'.$method), $params);
		}
		else
		{
			array_unshift_assoc($params, NULL, $method);
			return call_user_func_array(array($this, 'p_show'), $params);
		}
	}

	/**
	 * Exibe os registros na interface pública
	 */
	function p_index()
	{
		$this->p_list();
	}


	/**
	 * Exibe um registro na interface pública
	 */
	function p_show($label = '')
	{
		echo $label;
	}

	/**
	 * Prepara uma view genérica para exibir os registros em forma de tabela
	 */
	function p_list()
	{
		acesso('root', 1);
		$this->data['list_fields'] = array(
			'created' 	=> 'Data Cadastro',
			'updated' 	=> 'Data Edição',
			'title'		=> 'Título',
			'content' 	=> 'Conteúdo',
		);
		$this->data['title'] = 'Title';
		$this->data['page'] = 'admin/list';

		$dados = new Model();
		$this->data['data'] = $dados->get();

		$this->load->view('admin/index', $this->data);
	}

	/**
	 * Chama a função de edição, sem parâmetros
	 */
	function p_add()
	{
		$this->p_edit();
	}

	/**
	 * Cadastrar ou editar um registro
	 *
	 * @param string $label
	 */
	function p_edit($id = '')
	{

		$d = new Post();
		$this->data['data'] = $d->where('id', $id)->get();
		$this->data['page'] = 'admin/cadastro';

		if ($d->exists())
		{
			$this->data['title'] = 'Title edit';
			$this->data['url'] = $this->controller.'/edit/'.$id;
		}
		else
		{
			$this->data['title'] = 'Title add';
			$this->data['url'] = $this->controller.'/add';
		}

		if ($_POST)
		{
			$this->lang->load('form_validation', 'pt_BR');

			$info = $d->from_array($_POST, array(
				'title',
				'content',
			));
			if ($d->save($info))
			{
				$dir = str_replace('/', '\/', $this->config->item('img_directory'));
				preg_match_all('/'.$dir.'([^"]*)/', $_POST['content'], $matches);

				if (isset($matches[1]) and sizeof($matches[1]) > 0)//Se existir imagens, salvas no diretório do site, no corpo do conteúdo entra aqui
				{
					foreach ($matches[1] as $row) 
					{
						$i = new Image();
						$i->where('big', $row)->get();
						if ($i->exists()) 
						{
							$i->description = $d->title;
							$i->save($d);
						}
					}

					$destaque = new Image();
					$destaque->where('big', $matches[1][0])->get();
					if ($destaque->exists()) 
					{
						$d->image_id = $destaque->id;
						$d->save();
					}

					//Busca as imagens relacionadas com o item que não estão no array
					$i = new Image();
					$i->where_related_post('id', $id)->where_not_in('big', $matches[1])->get();
				}
				else //Senão exclui todas as imagens do banco relacionadas com o item
				{
					//Busca as imagens relacionadas com o item
					$i = new Image();
					$i->where_related_post('id', $id)->get();
				}

				if ($i->exists())
				{
					$i->delete_all();
				}
				$this->session->set_flashdata('success', 'Registro efetuado com sucesso!');
				redirect($this->controller.'/edit/'.$d->id);
			}
		}

		$d->load_extension('htmlform');
		$this->data['form_fields'] = array(
			'title',
			'content' => array(
				'template' => 'dmz_htmlform/row_redactor'
			),
		);

		$this->data['assets'] = array(
			'<link rel="stylesheet" href="'.assets_url('redactor/redactor.css').'" />',
			'<script src="'.assets_url('redactor/redactor.min.js').'"></script>',
			'<script src="'.assets_url('redactor/pt_br.js').'"></script>',
		);

		$this->load->view('admin/index', $this->data);
	}

	/**
	 * Excluir um registro
	 *
	 * @param string $label
	 */
	function p_delete($label = '')
	{
		if ( ! empty($label))
		{
			$d = new Model();
			$d->where('label', $label)->get();
			if ($d->delete())
			{
				$this->session->set_flashdata('success', 'Registro excluído com sucesso.');
			}
			else
			{
				$this->session->set_flashdata('fail', 'Ocorreu um erro ao excluir o registro. Por favor tente novamente ou contate o administrador do sistema.');
			}
		}

		redirect($this->controller);
	}

}

// End of file